<?php
namespace MineCloudvod\LMS;

defined( 'ABSPATH' ) || exit;

class Options{
    
    public function __construct() {
        add_action( 'mcv_add_admin_options_before_purchase', array( $this, 'course_admin_options' ) );

        add_filter( 'mcv_lms_course_base_slug', [$this, 'course_base_slug'] );
        add_filter( 'mcv_lms_lesson_base_slug', [$this, 'lesson_base_slug'] );
    }

    public function course_base_slug( $slug ){
        if(MINECLOUDVOD_SETTINGS['mcv_lms_general']['course_slug'] ?? false){
            return MINECLOUDVOD_SETTINGS['mcv_lms_general']['course_slug'];
        }
        return $slug;
    }
    public function lesson_base_slug( $slug ){
        if(MINECLOUDVOD_SETTINGS['mcv_lms_general']['lesson_slug'] ?? false){
            return MINECLOUDVOD_SETTINGS['mcv_lms_general']['lesson_slug'];
        }
        return $slug;
    }

    public function course_admin_options(){
        $prefix = 'mcv_settings';
        
        \MCSF::createSection( $prefix, array(
            'id'      => 'mcv_lms',
            'title'       => __('Mine LMS', 'mine-cloudvod'),
            'icon'        => 'fas fa-graduation-cap',
            'description' => ''
        ));

        \MCSF::createSection( $prefix, array(
            'parent'      => 'mcv_lms',
            'title'       => __('General settings', 'mine-cloudvod'),
            'icon'        => 'fas fa-book-reader',
            'description' => '',
            'fields'      => array(
                array(
                    'id'        => 'mcv_lms_general',
                    'type'      => 'fieldset',
                    'fields'    => array(
                        array(
                            'id'            => 'template',
                            'title'         => __('Active Template', 'mine-cloudvod'),
                            'type'          => 'select',
                            /**
                             * 过滤器 mcv_lms_templates 
                             */
                            'options'       =>  apply_filters('mcv_lms_templates', [
                                                    // 'default' => __('Default', 'mine-cloudvod'),
                                                    'ketang' => __('KeTang', 'mine-cloudvod'),
                                                ]),
                            'attributes' => array(
                              'style'    => 'min-width: 100px;'
                            ),
                            'default'     => 'ketang',
                        ),
                        array(
                            'type'    => 'submessage',
                            'style'   => 'success',
                            'content' => __('After change the slug, please enter the Setting - Permalink, and just click the save button, it will take effect.', 'mine-cloudvod'),
                        ),
                        array(
                            'id'            => 'course_slug',
                            'title'         => __('Course Slug', 'mine-cloudvod'),
                            'type'          => 'text',
                            'default'     => MINECLOUDVOD_LMS['course_post_type'],
                        ),
                        array(
                            'id'            => 'lesson_slug',
                            'title'         => __('Lesson Slug', 'mine-cloudvod'),
                            'type'          => 'text',
                            'default'     => MINECLOUDVOD_LMS['lesson_post_type'],
                        ),
                        array(
                            'id'            => 'course_permalink',
                            'title'         => __('Course Permalink', 'mine-cloudvod'),
                            'type'          => 'select',
                            'options'       => [
                                'postname'  => '/'.(MINECLOUDVOD_SETTINGS['mcv_lms_general']['course_slug']??MINECLOUDVOD_LMS['course_post_type']).'/%postname%/',
                                'postid'  => '/'.(MINECLOUDVOD_SETTINGS['mcv_lms_general']['course_slug']??MINECLOUDVOD_LMS['course_post_type']).'/%post_id%/',
                            ],
                            'default'     => 'postname',
                        ),
                        array(
                            'id'            => 'lesson_permalink',
                            'title'         => __('Lesson Permalink', 'mine-cloudvod'),
                            'type'          => 'select',
                            'options'       => [
                                'postname'  => '/'.(MINECLOUDVOD_SETTINGS['mcv_lms_general']['course_slug']??MINECLOUDVOD_LMS['course_post_type']).'/%postname%/'.(MINECLOUDVOD_SETTINGS['mcv_lms_general']['lesson_slug']??MINECLOUDVOD_LMS['lesson_post_type']).'/%postname%/',
                                'postid'  => '/'.(MINECLOUDVOD_SETTINGS['mcv_lms_general']['lesson_slug']??MINECLOUDVOD_LMS['lesson_post_type']).'/%post_id%/',
                            ],
                            'default'     => 'postname',
                        ),
                        array(
                            'id'            => 'wide_size',
                            'title'         => __('LMS Width', 'mine-cloudvod'),
                            'type'          => 'text',
                            'default'       => '1200px',
                        ),
                    ),
                ),
            )
        ));

        \MCSF::createSection( $prefix, [
            'parent'      => 'mcv_lms',
            'title'       => __('Course', 'mine-cloudvod'),
            'icon'        => 'fas fa-book-reader',
            'description' => '',
            'fields'      => [
                [
                    'id'        => 'mcv_lms_course',
                    'type'      => 'fieldset',
                    'fields'    => [
                        [
                            'id'        => 'archive',
                            'title'     => __('Archive Meta', 'mine-cloudvod'),
                            'type'      => 'fieldset',
                            'fields'    => [
                                [
                                    'id'            => 'notitle',
                                    'title'         => __('Add title tag', 'mine-cloudvod'),
                                    'desc'          => __('Enable it, if the archive page has no title.', 'mine-cloudvod'),
                                    'type'          => 'switcher',
                                    'text_on'       => __('Enable', 'mine-cloudvod'),
                                    'text_off'      => __('Disable', 'mine-cloudvod'),
                                    'default'       => false,
                                ],
                                [
                                    'id'            => 'title',
                                    'title'         => __('Title', 'mine-cloudvod'),
                                    'type'          => 'text',
                                    'desc'       => __('Courses Title', 'mine-cloudvod'),
                                ],
                                [
                                    'id'            => 'keywords',
                                    'title'         => __('Keywords', 'mine-cloudvod'),
                                    'type'          => 'text',
                                    'desc'       => __('Courses Keywords', 'mine-cloudvod'),
                                ],
                                [
                                    'id'            => 'desc',
                                    'title'         => __('Description', 'mine-cloudvod'),
                                    'type'          => 'text',
                                    'desc'       => __('Courses Description', 'mine-cloudvod'),
                                ],
                            ]
                        ],
                        [
                            'id'        => 'details',
                            'title'     => __('Details', 'mine-cloudvod'),
                            'type'      => 'fieldset',
                            'fields'    => [
                                [
                                    'id'            => 'lesson_num',
                                    'title'         => __('Number of Lesson', 'mine-cloudvod'),
                                    'type'          => 'switcher',
                                    'text_on'       => __('Show', 'mine-cloudvod'),
                                    'text_off'      => __('Hide', 'mine-cloudvod'),
                                    'default'       => true,
                                ],
                                [
                                    'id'            => 'hours',
                                    'title'         => __('Hours of Course', 'mine-cloudvod'),
                                    'type'          => 'switcher',
                                    'text_on'       => __('Show', 'mine-cloudvod'),
                                    'text_off'      => __('Hide', 'mine-cloudvod'),
                                    'default'       => true,
                                ],
                                [
                                    'id'            => 'student_num',
                                    'title'         => __('Number of Students', 'mine-cloudvod'),
                                    'type'          => 'switcher',
                                    'text_on'       => __('Show', 'mine-cloudvod'),
                                    'text_off'      => __('Hide', 'mine-cloudvod'),
                                    'default'       => true,
                                ],
                                [
                                    'id'            => 'update',
                                    'title'         => __('Date Updated', 'mine-cloudvod'),
                                    'type'          => 'switcher',
                                    'text_on'       => __('Show', 'mine-cloudvod'),
                                    'text_off'      => __('Hide', 'mine-cloudvod'),
                                    'default'       => true,
                                ],
                                [
                                    'id'            => 'difficulty',
                                    'title'         => __('Course Difficulty', 'mine-cloudvod'),
                                    'type'          => 'switcher',
                                    'text_on'       => __('Show', 'mine-cloudvod'),
                                    'text_off'      => __('Hide', 'mine-cloudvod'),
                                    'default'       => true,
                                ],
                                [
                                    'id'            => 'status',
                                    'title'         => __('Course Status', 'mine-cloudvod'),
                                    'type'          => 'switcher',
                                    'text_on'       => __('Show', 'mine-cloudvod'),
                                    'text_off'      => __('Hide', 'mine-cloudvod'),
                                    'default'       => true,
                                ],
                            ]
                        ],
                        [
                            'id'            => 'pagenum',
                            'title'         => __('Courses Per Page', 'mine-cloudvod'),
                            'type'          => 'text',
                            'default'       => 16,
                        ],
                        [
                            'id'            => 'filter',
                            'title'         => __('Course Filter', 'mine-cloudvod'),
                            'type'          => 'fieldset',
                            'fields'        =>[
                                [
                                    'id'            => 'status',
                                    'title'         => __('Status', 'mine-cloudvod'),
                                    'type'          => 'switcher',
                                    'text_on'       => __('Enable', 'mine-cloudvod'),
                                    'text_off'      => __('Disable', 'mine-cloudvod'),
                                    'default'       => true,
                                ],
                                [
                                    'id'            => 'items',
                                    'type'          => 'select',
                                    'title'         => __( 'Filter with', 'mine-cloudvod' ),
                                    'chosen'        => true,
                                    'multiple'      => true,
                                    'sortable'      => true,
                                    'options'       => [
                                        'category'  => __( 'Category', 'mine-cloudvod' ),
                                        'tag'       => __( 'Tag', 'mine-cloudvod' ),
                                        'difficulty'=> __( 'Difficulty', 'mine-cloudvod' ),
                                        'mode'      => __( 'Access Mode', 'mine-cloudvod' ),
                                    ],
                                    'default'    => array( 'category', 'difficulty', 'mode' ),
                                    'dependency' => [ 'status', '==', true ]
                                ],
                            ]
                        ],
                        [
                            'id'            => 'showCatelogInDetail',
                            'title'         => __('Show Catelog In Detail', 'mine-cloudvod'),
                            'type'          => 'switcher',
                            'text_on'       => __('Show', 'mine-cloudvod'),
                            'text_off'      => __('Hide', 'mine-cloudvod'),
                            'default'       => false,
                        ],
                        [
                            'id'            => 'backend',
                            'title'         => __('Backend options', 'mine-cloudvod'),
                            'type'          => 'fieldset',
                            'fields'        =>[
                                [
                                    'id'            => 'player',
                                    'title'         => __('Default player of lesson import', 'mine-cloudvod'),
                                    'type'          => 'radio',
                                    'options'       => [
                                        '1' => 'Aliplayer',
                                        '2' => 'Dplayer',
                                    ],
                                    'default'       => '1',
                                    'inline'        => true,
                                ],
                            ]
                        ],
                    ],
                ],
            ]
        ]);
    }
}